package body Main_Window is

	function WindowProc(Window: HWND; Message: UINT; W: WPARAM; L: LPARAM) return LRESULT is
	
		function GetState return Window_State_Access is
			function Cast is new Ada.Unchecked_Conversion(INT, Window_State_Access);
		begin
			return Cast(GetWindowLong(Window, GWL_USERDATA));
		end;
	
		State: Window_State_Access;
	begin
		case Message is
		when WM_CREATE =>
			State := new Window_State;
			declare
				SubMenu: HMENU;
			begin
				State.Menu := CreateMenu;
				SubMenu := CreatePopupMenu;
				AppendMenu(State.Menu, MF_POPUP, SubMenu, Addr("&File" & Nul));
				AppendMenu(SubMenu, MF_STRING, ID_EXIT, Addr("E&xit" & Nul));
				SubMenu := CreatePopupMenu;
				AppendMenu(State.Menu, MF_POPUP, SubMenu, Addr("&Edit" & Nul));
				declare
					use Ada.Strings.Unbounded;
					Data: aliased WIN32_FIND_DATA;
					H: HANDLE := FindFirstFile(Addr("*.txt" & Nul), Data'Access);
					I: Integer := ID_CONVERT_BASE;
				begin
					if H /= INVALID_HANDLE_VALUE then
						loop
							declare
								S : String := To_String(Data.FileName);
							begin
								AppendMenu(SubMenu, MF_STRING, DWORD(I), Addr(S(1..S'Length - 4) & Windows.Nul));
								State.Modes(I) := To_Unbounded_String(S);
							end;
							I := I + 1;
							exit when not FindNextFile(H, Data'Access);
						end loop;
						FindClose(H);
					end if;
				end;
				SubMenu := CreatePopupMenu;
				AppendMenu(State.Menu, MF_POPUP, SubMenu, Addr("&Help" & Nul));
				AppendMenu(SubMenu, MF_STRING, ID_ABOUT, Addr("&About..." & Nul));
				SetMenu(Window, State.Menu);
			end;
			State.Edit := CreateWindowEx(WS_EX_CLIENTEDGE, Addr("EDIT" & Nul), Addr("" & Nul),
					WS_VISIBLE or WS_CHILD or WS_VSCROLL or WS_HSCROLL or
					ES_MULTILINE or ES_AUTOVSCROLL or ES_AUTOHSCROLL or ES_WANTRETURN,
					0, 0, 100, 100, Window, null, Retrieve_hInstance, System.Null_Address);
			declare
				function Cast is new Ada.Unchecked_Conversion(Window_State_Access, LONG);
			begin
				SetWindowLong(Window, GWL_USERDATA, Cast(State));
			end;
			return DefWindowProc(Window, Message, W, L);
		when WM_DESTROY =>
			State := GetState;
			SetMenu(Window, null);
			DestroyMenu(State.Menu);
			PostQuitMessage(0);
			return DefWindowProc(Window, Message, W, L);
		when WM_SETFOCUS =>
			State := GetState;
			SetFocus(State.Edit);
			return DefWindowProc(Window, Message, W, L);
		when WM_SIZE =>
			State := GetState;
			if State /= null then
				declare
					R: aliased RECT;
				begin
					GetClientRect(Window, R'Access);
					MoveWindow(State.Edit, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top, BOOL_TRUE);
				end;
			end if;
			return DefWindowProc(Window, Message, W, L);
		when WM_COMMAND =>
			case Integer(W) is
			when ID_EXIT =>
				PostMessage(Window, WM_SYSCOMMAND, SC_CLOSE, 0);
			when ID_ABOUT =>
				MessageBox(Window, Addr("Written in Ada by Demoonlit" & Nul), Addr("src2html" & Nul), MB_ICONINFORMATION);
			when Mode_Table'Range =>
				State := GetState;
				declare
					use Ada.Strings.Unbounded;
					Buffer_Length: Integer := Integer(GetWindowTextLength(State.Edit)) + 1;
					Text: String(1..Buffer_Length);
					pragma Warnings(Off, Text);
					Length: Integer := Integer(GetWindowText(State.Edit, Addr(Text), INT(Buffer_Length)));
					New_Text: String := Replace(Text(1..Length), To_String(State.Modes(Integer(W)))) & Nul;
				begin
					SetWindowText(State.Edit, Addr(New_Text));
				end;
			when others =>
				null;
			end case;
			return DefWindowProc(Window, Message, W, L);
		when others =>
			return DefWindowProc(Window, Message, W, L);
		end case;
	end WindowProc;

end Main_Window;
